<?php

namespace App\Repository;

use App\Entity\Client;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;

/**
 * @extends ServiceEntityRepository<Client>
 */
class ClientRepository extends AbstractRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Client::class);
    }

    public function findClientsByOrganizationalUnitAndDescendants(int $organizationalUnitId): array
    {
        $query = $this->getEntityManager()->createQuery(
            'SELECT o.path
         FROM App\Entity\OrganizationalUnit o
         WHERE o.id = :id'
        )->setParameter('id', $organizationalUnitId);

        $result = $query->getOneOrNullResult();

        if (!$result) {
            return [];
        }

        $path = $result['path'] . '/%';

        $query = $this->getEntityManager()->createQuery(
            'SELECT o.id
         FROM App\Entity\OrganizationalUnit o
         WHERE o.id = :id OR o.path LIKE :path'
        )
            ->setParameter('id', $organizationalUnitId)
            ->setParameter('path', $path);

        $unitIds = array_column($query->getArrayResult(), 'id');

        $query = $this->getEntityManager()->createQuery(
            'SELECT c
         FROM App\Entity\Client c
         WHERE c.organizationalUnit IN (:unitIds)'
        )
            ->setParameter('unitIds', $unitIds);

        return $query->getResult();
    }
}
