<?php

namespace App\Service;

use App\Entity\Client;
use App\Entity\OperativeSystem;
use App\Entity\Partition;
use App\Model\PartitionTypes;
use Doctrine\ORM\EntityManagerInterface;
use Exception;

class CreatePartitionService
{
    public function __construct(
        protected EntityManagerInterface $entityManager
    )
    {
    }

    public function __invoke(array $data, Client $clientEntity): void
    {
        $currentPartitions = $this->entityManager->getRepository(Partition::class)
            ->findBy(['client' => $clientEntity]);

        $receivedPartitions = [];

        foreach ($data['cfg'] as $cfg) {
            if (!isset($cfg['disk'], $cfg['par'], $cfg['tam'], $cfg['uso'], $cfg['fsi'])) {
                continue;
            }

            $partitionEntity = $this->entityManager->getRepository(Partition::class)
                ->findOneBy(['client' => $clientEntity, 'diskNumber' => $cfg['disk'], 'partitionNumber' => $cfg['par']]);

            if (!$partitionEntity) {
                $partitionEntity = new Partition();
            }

            if (isset($cfg['soi']) && $cfg['soi'] !== '') {
                $operativeSystem = $this->entityManager->getRepository(OperativeSystem::class)
                    ->findOneBy(['name' => $cfg['soi']]);

                if (!$operativeSystem) {
                    $operativeSystem = new OperativeSystem();
                    $operativeSystem->setName($cfg['soi']);
                    $this->entityManager->persist($operativeSystem);
                }
                $partitionEntity->setOperativeSystem($operativeSystem);
            }

            $partitionEntity->setClient($clientEntity);
            $partitionEntity->setDiskNumber($cfg['disk']);
            $partitionEntity->setPartitionNumber($cfg['par']);
            $partitionEntity->setSize($cfg['tam']);

            if (isset($cfg['cpt']) && $cfg['cpt'] !== '') {
                $decimalValue = hexdec($cfg['cpt']);
                $partitionType = PartitionTypes::getPartitionType($decimalValue);

                if ($partitionType) {
                    $partitionEntity->setPartitionCode($partitionType['name']);
                }
            } else {
                $partitionEntity->setPartitionCode(PartitionTypes::getPartitionType(0)['name']);
            }

            $partitionEntity->setFilesystem($cfg['fsi']);
            $partitionEntity->setMemoryUsage(((int) $cfg['uso']) * 100);
            $this->entityManager->persist($partitionEntity);

            $receivedPartitions[] = ['disk' => $cfg['disk'], 'partition' => $cfg['par']];
        }

        foreach ($currentPartitions as $currentPartition) {
            $exists = false;

            foreach ($receivedPartitions as $receivedPartition) {
                if ($currentPartition->getDiskNumber() == $receivedPartition['disk'] &&
                    $currentPartition->getPartitionNumber() == $receivedPartition['partition']) {
                    $exists = true;
                    break;
                }
            }

            if (!$exists) {
                $this->entityManager->remove($currentPartition);
            }
        }

        $this->entityManager->flush();
    }
}